#
# Copyright (C) 2019 - present Instructure, Inc.
#
# This file is part of Canvas.
#
# Canvas is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, version 3 of the License.
#
# Canvas is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.

require_relative './page_objects/student_assignment_page_v2'
require_relative '../common'

describe 'assignments' do
  include_context "in-process server selenium tests"

  context 'as a student' do

    before(:once) do
      Account.default.enable_feature!(:assignments_2)
      course_with_student(course: @course, active_all: true)
      @assignment = @course.assignments.create!(
        name: 'assignment',
        due_at: 5.days.ago,
        points_possible: 10,
        submission_types: 'online_upload'
      )
      preload_graphql_schema
    end

    before(:each) do
      user_session(@student)
      StudentAssignmentPageV2.visit(@course, @assignment)
    end

    it 'should show available checkmark stepper' do
      expect(StudentAssignmentPageV2.checkmark_icon).to be_displayed
    end

    it 'should show assignment title' do
      expect(StudentAssignmentPageV2.assignment_title(@assignment.title)).to_not be_nil
    end

    it 'available assignment should show details toggle' do
      expect(StudentAssignmentPageV2.details_toggle).to be_displayed
    end

    it 'should show assignment group link' do
      expect(StudentAssignmentPageV2.assignment_group_link).to be_displayed
    end

    it 'should show assignment due date' do
      expect(StudentAssignmentPageV2.due_date_css(@assignment.due_at)).to_not be_nil
    end

    it 'should show how many points possible the assignment is worth' do
      expect(StudentAssignmentPageV2.points_possible_css(@assignment.points_possible)).to_not be_nil
    end

    it 'available assignment should show content tablist' do
      expect(StudentAssignmentPageV2.content_tablist).to be_displayed
    end

  end
end
